@@ -3,7 +3,7 @@
 #
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2020 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -23,34 +23,17 @@
 
 from . import utils
 import sys
-import mailbox
 import email
 import email.errors
-import io
 import glob
 import os
-import re
-import urllib.request, urllib.parse, urllib.error
+import urllib.parse
 import textwrap
 # SOAP interface to Debian BTS
 import debianbts
 from collections import defaultdict
 
 from . import checkversions
-from .exceptions import (
-    NoNetwork,
-    QuertBTSError,
-)
-from .urlutils import open_url
-
-
-def msgfactory(fp):
-    try:
-        return email.message_from_file(fp)
-    except email.errors.MessageParseError:
-        # Don't return None since that will
-        # stop the mailbox iterator
-        return ''
 
 
 class Error(Exception):
@@ -60,30 +43,30 @@ class Error(Exception):
 # Severity levels
 SEVERITIES = {
     'critical': """makes unrelated software on the system (or the
-    whole system) break, or causes serious data loss, or introduces a
-    security hole on systems where you install the package.""",
+        whole system) break, or causes serious data loss, or introduces a
+        security hole on systems where you install the package.""",
     'grave': """makes the package in question unusable by most or all users,
-    or causes data loss, or introduces a security hole allowing access
-    to the accounts of users who use the package.""",
+        or causes data loss, or introduces a security hole allowing access
+        to the accounts of users who use the package.""",
     'serious': """is a severe violation of Debian policy (that is,
-    the problem is a violation of a 'must' or 'required' directive);
-    may or may not affect the usability of the package.  Note that non-severe
-    policy violations may be 'normal,' 'minor,' or 'wishlist' bugs.
-    (Package maintainers may also designate other bugs as 'serious' and thus
-    release-critical; however, end users should not do so.). For the canonical
-    list of issues deserving a serious severity you can refer to this webpage:
-    http://release.debian.org/testing/rc_policy.txt .""",
+        the problem is a violation of a 'must' or 'required' directive);
+        may or may not affect the usability of the package.  Note that non-severe
+        policy violations may be 'normal,' 'minor,' or 'wishlist' bugs.
+        (Package maintainers may also designate other bugs as 'serious' and thus
+        release-critical; however, end users should not do so.). For the canonical
+        list of issues deserving a serious severity you can refer to this webpage:
+        https://release.debian.org/testing/rc_policy.txt .""",
     'important': """a bug which has a major effect on the usability
-    of a package, without rendering it completely unusable to
-    everyone.""",
+        of a package, without rendering it completely unusable to
+        everyone.""",
     'does-not-build': """a bug that stops the package from being built
-    from source.  (This is a 'virtual severity'.)""",
+        from source.  (This is a 'virtual severity'.)""",
     'normal': """a bug that does not undermine the usability of the
-    whole package; for example, a problem with a particular option or
-    menu item.""",
+        whole package; for example, a problem with a particular option or
+        menu item.""",
     'minor': """things like spelling mistakes and other minor
-    cosmetic errors that do not affect the core functionality of the
-    package.""",
+        cosmetic errors that do not affect the core functionality of the
+        package.""",
     'wishlist': "suggestions and requests for new features.",
 }
 
@@ -91,25 +74,25 @@ SEVERITIES = {
 JUSTIFICATIONS = {
     'critical': (
         ('breaks unrelated software', """breaks unrelated software on the system
-    (packages that have a dependency relationship are not unrelated)"""),
+            (packages that have a dependency relationship are not unrelated)"""),
         ('breaks the whole system', """renders the entire system unusable (e.g.,
-    unbootable, unable to reach a multiuser runlevel, etc.)"""),
+            unbootable, unable to reach a multiuser runlevel, etc.)"""),
         ('causes serious data loss', """causes loss of important, irreplaceable
-    data"""),
+            data"""),
         ('root security hole', """introduces a security hole allowing access to
-    root (or another privileged system account), or data normally
-    accessible only by such accounts"""),
+            root (or another privileged system account), or data normally
+            accessible only by such accounts"""),
         ('unknown', """not sure, or none of the above"""),
     ),
     'grave': (
         ('renders package unusable', """renders the package unusable, or mostly
-    so, on all or nearly all possible systems on which it could be installed
-    (i.e., not a hardware-specific bug); or renders package uninstallable
-    or unremovable without special effort"""),
+            so, on all or nearly all possible systems on which it could be installed
+            (i.e., not a hardware-specific bug); or renders package uninstallable
+            or unremovable without special effort"""),
         ('causes non-serious data loss', """causes the loss of data on the system
-    that is unimportant, or restorable without resorting to backup media"""),
+            that is unimportant, or restorable without resorting to backup media"""),
         ('user security hole', """introduces a security hole allowing access to
-    user accounts or data not normally accessible"""),
+            user accounts or data not normally accessible"""),
         ('unknown', """not sure, or none of the above"""),
     )
 }
@@ -208,6 +191,28 @@ progenyother = {
 }
 
 
+def check_package_info(package):
+    # check dpkg database
+    info = utils.get_package_status(package)
+    if info[1]:
+        return info
+
+    # check apt cache
+    info = utils.get_package_status(package, avail=True)
+    if info[1]:
+        return info
+
+    # check if this is a source package and get the info for one of
+    # its binary packages
+    pkgs = utils.get_source_package(package, only_source=True)
+    if pkgs:
+        info = check_package_info(pkgs[0][0])
+        if info[1]:
+            return info
+
+    return None
+
+
 def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
     body = reason = archs = section = priority = ''
     suite = 'unstable'
@@ -244,15 +249,10 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
             ui.log_message('You seem to want to report a generic bug, not request a removal\n')
             return
 
-        ui.log_message('Checking status database...\n')
-        info = utils.get_package_status(package)
-        available = info[1]
+        ui.log_message('Checking package information...\n')
+        info = check_package_info(package)
 
         query = False
-        if not available:
-            info = utils.get_source_package(package)
-            if info:
-                info = utils.get_package_status(info[0][0])
 
         if not info:
             cont = ui.select_options(
@@ -265,8 +265,8 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
             section, priority = info[16], info[10]
 
     if tag == 'override':
-        pseudos.append('X-Debbugs-Cc: debian-boot@lists.debian.org')
-        ui.log_message('Your report will be carbon-copied to debian-boot.\n')
+        pseudos.append('User: ftp.debian.org@packages.debian.org')
+        pseudos.append('Usertags: override')
 
         # we handle here the override change request
         new_section = ui.menu('Select the new section', {
@@ -274,7 +274,8 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
             'debian-installer': "", 'debug': "", 'devel': "", 'doc': "",
             'editors': "", 'education': "", 'electronics': "",
             'embedded': "", 'fonts': "", 'games': "", 'gnome': "",
-            'gnu-r': "", 'gnustep': "", 'graphics': "", 'hamradio': "",
+            'gnu-r': "", 'gnustep': "", 'golang': "",
+            'graphics': "", 'hamradio': "",
             'haskell': "", 'httpd': "", 'interpreters': "",
             'introspection': "", 'java': "", "javascript": "",
             'kde': "", 'kernel': "",
@@ -311,6 +312,10 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
                                 "exiting...")
                 sys.exit(1)
 
+        if new_priority != priority:
+            pseudos.append('X-Debbugs-Cc: debian-boot@lists.debian.org')
+            ui.log_message('Your report will be carbon-copied to debian-boot.\n')
+
         arch_section = ui.menu('Is this request for an archive section other than "main"?', {
             'main': "",
             'contrib': "",
@@ -327,15 +332,16 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
     else:
         # we handle here the removal requests
         suite = ui.menu('Is the removal to be done in a suite other than'
-                        ' "unstable"?  Don\'t select anything for "unstable"', {
-                            'oldstable': "Old stable.",
-                            'oldstable-proposed-updates': "Old stable proposed updates.",
-                            'stable': "Stable.",
-                            'stable-proposed-updates': "Stable proposed updates.",
+                        ' "unstable"?', {
+                            'oldstable': "Old stable",
+                            'oldstable-proposed-updates': "Old stable proposed updates",
+                            'stable': "Stable",
+                            'stable-proposed-updates': "Stable proposed updates",
                             'testing': "Testing only (NOT unstable)",
                             'testing-proposed-updates': "Testing proposed updates",
-                            'experimental': "Experimental.",
-                        }, 'Choose the suite: ', empty_ok=True)
+                            'unstable': "Unstable",
+                            'experimental': "Experimental",
+                        }, 'Choose the suite: ', default='unstable', empty_ok=True)
         if not suite:
             suite = 'unstable'
 
@@ -379,8 +385,8 @@ def handle_debian_ftp(package, bts, ui, fromaddr, timeout, online=True, http_pro
                 subject = 'RM: %s -- %s; %s' % (package, tag, reason)
 
         if suite == 'testing':
-            ui.long_message('Please use release.debian.org pseudo-package and '
-                            'report a bug there.')
+            ui.long_message('Removals from testing are handled by the release team. '
+                            'Please file a bug against the release.debian.org pseudo-package.')
             sys.exit(1)
 
     return (subject, severity, headers, pseudos, body, query)
@@ -438,15 +444,10 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
             ui.log_message('You seem to want to report a generic bug.\n')
             return
 
-        ui.log_message('Checking status database...\n')
-        info = utils.get_package_status(package)
-        available = info[1]
+        ui.log_message('Checking package information...\n')
+        info = check_package_info(package)
 
         query = False
-        if not available:
-            info = utils.get_source_package(package)
-            if info:
-                info = utils.get_package_status(info[0][0])
 
         if not info:
             cont = ui.select_options(
@@ -498,8 +499,8 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
                 ui.long_message('No architecture specified, skipping...')
 
     if tag == 'binnmu':
-        suite = ui.menu("For which suite are you requesting this binNMU?"
-                        "  Don't select anything for \"unstable\"", {
+        suite = ui.menu("For which suite are you requesting this binNMU?",
+                        {
                             stable: "",
                             stable_backports: "",
                             stable_security: "",
@@ -507,8 +508,10 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
                             oldstable_backports: "",
                             oldstable_security: "",
                             testing: "",
+                            'unstable': "",
                             'experimental': "",
-                        }, 'Choose the suite: ', empty_ok=True)
+                        },
+                        'Choose the suite: ', default='unstable', empty_ok=True)
         if not suite:
             suite = 'unstable'
 
@@ -539,7 +542,7 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
                         'ensure that the "old" regex does not match the "new" packages.  '
                         'Example: old="/libapt-pkg4.10|libapt-inst1.2/ libept1" '
                         'new="/libapt-pkg4.12|libapt-inst1.5|libept1.4.12/". For further '
-                        'reference, please refer to http://ben.debian.net/ .')
+                        'reference, please refer to https://debian.pages.debian.net/ben/ .')
 
         prompt = 'Please enter old binary package name of the library (or a regex matching it):'
         tfrom = ui.get_string(prompt)
@@ -552,7 +555,8 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
             # Compute a ben file from this.
 
             # (quote if x does not start with a "/")
-            quote = lambda x: (x[0] == '/' and x) or '"%s"' % x
+            def quote(x):
+                return (x[0] == '/' and x) or '"%s"' % x
 
             listbad = [quote(x) for x in tfrom.strip().split()]
             listgood = [quote(x) for x in tto.strip().split()]
@@ -641,8 +645,16 @@ def handle_debian_release(package, bts, ui, fromaddr, timeout, online=True, http
 
                 [ Other info ]
                 (Anything else the release team should know.)
-                """ % (package, package, version))
+                """)
     elif tag == 'rm':
+        cont = ui.select_options("Is the removal you are asking for really to be done in "
+                                 "stable/testing and not in unstable/experimental?",
+                                 "Yn", {"y": "Yes, the package should be removed from stable/testing.",
+                                        "n": "No, I want to request removal from unstable or experimental."})
+        if cont != "y":
+            ui.long_message('Removal requests for unstable and experimental are handled by the FTP team. '
+                            'Please file a bug against the ftp.debian.org pseudo-package.')
+            sys.exit(1)
         subject = 'RM: %s/%s' % (package, version)
         body = '(explain the reason for the removal here)\n'
 
@@ -653,7 +665,7 @@ itp_template = textwrap.dedent("""\
     * Package name    : %(package)s
       Version         : x.y.z
       Upstream Author : Name <somebody@example.org>
-    * URL             : http://www.example.org/
+    * URL             : https://www.example.org/
     * License         : (GPL, LGPL, BSD, MIT/X, etc.)
       Programming Lang: (C, C++, C#, Perl, Python, etc.)
       Description     : %(short_desc)s
@@ -681,11 +693,23 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
                   'things mean anything to you, or you are trying to report '
                   'a bug in an existing package, please press Enter to '
                   'exit reportbug.)', {
-                      'O': "The package has been `Orphaned'. It needs a new maintainer as soon as possible.",
-                      'RFA': "This is a `Request for Adoption'. Due to lack of time, resources, interest or something similar, the current maintainer is asking for someone else to maintain this package. They will maintain it in the meantime, but perhaps not in the best possible way. In short: the package needs a new maintainer.",
-                      'RFH': "This is a `Request For Help'. The current maintainer wants to continue to maintain this package, but they need some help to do this because their time is limited or the package is quite big and needs several maintainers.",
-                      'ITP': "This is an `Intent To Package'. Please submit a package description along with copyright and URL in such a report.",
-                      'RFP': "This is a `Request For Package'. You have found an interesting piece of software and would like someone else to maintain it for Debian. Please submit a package description along with copyright and URL in such a report.",
+                      'O': "The package has been `Orphaned'. "
+                           "It needs a new maintainer as soon as possible.",
+                      'RFA': "This is a `Request for Adoption'. "
+                             "Due to lack of time, resources, interest or something similar, "
+                             "the current maintainer is asking for someone else to maintain this package. "
+                             "They will maintain it in the meantime, but perhaps not in the best possible way. "
+                             "In short: the package needs a new maintainer.",
+                      'RFH': "This is a `Request For Help'. "
+                             "The current maintainer wants to continue to maintain this package, "
+                             "but they need some help to do this because their time is limited "
+                             "or the package is quite big and needs several maintainers.",
+                      'ITP': "This is an `Intent To Package'. "
+                             "Please submit a package description along with copyright and URL in such a report.",
+                      'RFP': "This is a `Request For Package'. "
+                             "You have found an interesting piece of software and would like someone "
+                             "else to maintain it for Debian. Please submit a package description "
+                             "along with copyright and URL in such a report.",
                   }, 'Choose the request type: ', empty_ok=True)
     if not tag:
         ui.long_message('To report a bug in a package, use the name of the package, not wnpp.\n')
@@ -704,9 +728,9 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
             ui.long_message('Invalid package name')
             package = ""
 
-    ui.log_message('Checking status database...\n')
-    info = utils.get_package_status(package, avail=True)
-    available = info[1]
+    ui.log_message('Checking package information...\n')
+    info = check_package_info(package)
+    available = info and info[1]
 
     severity = 'normal'
     if tag in ('ITP', 'RFP'):
@@ -742,10 +766,6 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
     elif tag in ('O', 'RFA', 'RFH'):
         severity = 'normal'
         query = False
-        if not available:
-            info = utils.get_source_package(package)
-            if info:
-                info = utils.get_package_status(info[0][0])
 
         if not info:
             cont = ui.select_options(
@@ -780,6 +800,8 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
                     'The package description is:\n') % (orphstr, package)
             body = body + long_desc + '\n'
 
+        pseudos.append(f'Control: affects -1 src:{package}')
+
     if short_desc:
         subject = '%s: %s -- %s' % (tag, package, short_desc)
     else:
@@ -788,6 +810,115 @@ def handle_wnpp(package, bts, ui, fromaddr, timeout, online=True, http_proxy=Non
     return (subject, severity, headers, pseudos, body, query)
 
 
+def handle_installation_report(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
+    body = ''
+    headers = []
+    pseudos = []
+    query = True
+    severity = ''
+    subject = ''
+
+    bootmethod = (ui.get_string('How did you boot the installer (CD/DVD/USB/network/...)? ')
+                  or '<boot method (CD/DVD, USB stick, network, etc.>')
+
+    image = (ui.get_string('What image did you use to install? (If you can, give its URL and build date) ')
+             or '<Full URL to image you downloaded is best>')
+
+    machine = (ui.get_string('Describe your machine (manufacturer and type): ')
+               or '<Description of machine (manufacturer, type)>')
+
+    body = textwrap.dedent(f"""\
+        (Please provide enough information to help the Debian
+        maintainers evaluate the report efficiently - e.g., by filling
+        in the sections below.)
+
+        Boot method: {bootmethod}
+        Image version: {image}
+        Date: <Date and time of the install>
+
+        Machine: {machine}
+        Partitions: <df -Tl will do; the raw partition table is preferred>
+
+
+        Base System Installation Checklist:
+        [O] = OK, [E] = Error (please elaborate below), [ ] = didn't try it
+
+        Initial boot:           [ ]
+        Detect network card:    [ ]
+        Configure network:      [ ]
+        Detect media:           [ ]
+        Load installer modules: [ ]
+        Clock/timezone setup:   [ ]
+        User/password setup:    [ ]
+        Detect hard drives:     [ ]
+        Partition hard drives:  [ ]
+        Install base system:    [ ]
+        Install tasks:          [ ]
+        Install boot loader:    [ ]
+        Overall install:        [ ]
+
+        Comments/Problems:
+
+        <Description of the install, in prose, and any thoughts, comments
+              and ideas you had during the initial install.>
+
+
+        Please make sure that any installation logs that you think would
+        be useful are attached to this report. Please compress large
+        files using gzip.
+        """)
+
+    return (subject, severity, headers, pseudos, body, query)
+
+
+def handle_upgrade_report(package, bts, ui, fromaddr, timeout, online=True, http_proxy=None):
+    body = ''
+    headers = []
+    pseudos = []
+    query = True
+    severity = ''
+    subject = ''
+
+    body = textwrap.dedent("""\
+        (Please provide enough information to help the Debian
+        maintainers evaluate the report efficiently - e.g., by filling
+        in the sections below.)
+
+        My previous release is: <codename or version from which you are upgrading>
+        I am upgrading to: <codename or version of the release you are upgrading to>
+        Archive date: <Timestamp, available as project/trace/ftp-master.debian.org
+             on your mirror or .disk/info on your CD/DVD set>
+        Upgrade date: <Date and time of the upgrade>
+        uname -a before upgrade: <The result of running uname -a on a shell prompt>
+        uname -a after upgrade: <The result of running uname -a on a shell prompt>
+        Method: <How did you upgrade?  Which program did you use?>
+
+        Contents of /etc/apt/sources.list:
+
+
+        - Were there any non-Debian packages installed before the upgrade?  If
+          so, what were they?
+
+        - Was the system pre-update a 'pure' system only containing packages
+          from the previous release? If not, which packages were not from that
+          release?
+
+        - Did any packages fail to upgrade?
+
+        - Were there any problems with the system after upgrading?
+
+
+        Further Comments/Problems:
+
+
+        Please attach the output of "COLUMNS=200 dpkg -l" (or "env COLUMNS ...",
+        depending on your shell) from before and after the upgrade so that we
+        know what packages were installed on your system.
+        """)
+
+    return (subject, severity, headers, pseudos, body, query)
+
+
 def dpkg_infofunc():
     debarch = utils.get_arch()
     utsmachine = os.uname()[4]
@@ -820,31 +951,40 @@ def generic_infofunc():
 
 # Supported servers
 # Theoretically support for GNATS and Jitterbug could be added here.
-SYSTEMS = {'debian':
-           {'name': 'Debian', 'email': '%s@bugs.debian.org',
-            'btsroot': 'http://www.debian.org/Bugs/',
-            'otherpkgs': debother,
-            'nonvirtual': ['linux-image', 'kernel-image'],
-            'specials':
-                {'wnpp': handle_wnpp,
-                 'ftp.debian.org': handle_debian_ftp,
-                 'release.debian.org': handle_debian_release},
-            # Dependency packages
-            'deppkgs': ('gcc', 'g++', 'cpp', 'gcj', 'gpc', 'gobjc',
-                        'chill', 'gij', 'g77', 'python', 'python-base',
-                        'x-window-system-core', 'x-window-system'),
-            'cgiroot': 'https://bugs.debian.org/cgi-bin/',
-            'infofunc': debian_infofunc,
-            },
-           'ubuntu':
-               {'name': 'Ubuntu', 'email': 'ubuntu-users@lists.ubuntu.com',
-                'type': 'mailto',
-                'infofunc': ubuntu_infofunc,
-                },
-           'guug':
-               {'name': 'GUUG (German Unix User Group)',
-                'email': '%s@bugs.guug.de', 'query-dpkg': False},
-           }
+SYSTEMS = {
+    'debian': {
+        'name': 'Debian',
+        'email': '%s@bugs.debian.org',
+        'btsroot': 'http://www.debian.org/Bugs/',
+        'otherpkgs': debother,
+        'nonvirtual': ['linux-image', 'kernel-image'],
+        'specials': {
+            'wnpp': handle_wnpp,
+            'ftp.debian.org': handle_debian_ftp,
+            'release.debian.org': handle_debian_release,
+            'installation-reports': handle_installation_report,
+            'upgrade-reports': handle_upgrade_report,
+        },
+        # Dependency packages
+        'deppkgs': (
+            'gcc', 'g++', 'cpp', 'gcj', 'gpc', 'gobjc',
+            'chill', 'gij', 'g77', 'python', 'python-base',
+            'x-window-system-core', 'x-window-system'
+        ),
+        'cgiroot': 'https://bugs.debian.org/cgi-bin/',
+        'infofunc': debian_infofunc,
+    },
+    'ubuntu': {
+        'name': 'Ubuntu',
+        'email': 'ubuntu-users@lists.ubuntu.com',
+        'type': 'mailto',
+        'infofunc': ubuntu_infofunc,
+    },
+    'guug': {
+        'name': 'GUUG (German Unix User Group)',
+        'email': '%s@bugs.guug.de',
+        'query-dpkg': False},
+}
 
 CLASSES = {
     'sw-bug': 'The problem is a bug in the software or code.  For'
@@ -910,12 +1050,11 @@ def yn_bool(setting):
         return 'no'
 
 
-def cgi_report_url(system, number, archived=False, mbox=False, mboxmaint=False):
+def cgi_report_url(system, number, archived=False, mbox=False):
     root = SYSTEMS[system].get('cgiroot')
-    if root:
-        return '%sbugreport.cgi?bug=%d&archived=%s&mbox=%s&mboxmaint=%s' % (
-            root, number, archived, yn_bool(mbox), yn_bool(mboxmaint))
-    return None
+    if not root:
+        return None
+    return f"{root}bugreport.cgi?bug={number}&mboxmaint={yn_bool(mbox)}"
 
 
 def cgi_package_url(system, package, archived=False, source=False,
@@ -931,7 +1070,7 @@ def cgi_package_url(system, package, archived=False, source=False,
         query = {'pkg': package.lower()}
 
     query['repeatmerged'] = yn_bool(repeatmerged)
-    query['archived'] = yn_bool(archived)
+    query['archived'] = 'both' if archived else 'no'
 
     if version:
         query['version'] = str(version)
@@ -941,33 +1080,13 @@ def cgi_package_url(system, package, archived=False, source=False,
     return '%spkgreport.cgi?%s' % (root, qstr)
 
 
-# TODO: to be removed
-def package_url(system, package, mirrors=None, source=False,
-                repeatmerged=True):
-    btsroot = get_btsroot(system, mirrors)
-    package = urllib.parse.quote_plus(package.lower())
-    return btsroot + ('db/pa/l%s.html' % package)
-
-
-# TODO: to be removed
-def report_url(system, number, mirrors=None):
-
-    number = str(number)
-    if len(number) < 2:
-        return None
-    btsroot = get_btsroot(system, mirrors)
-    return btsroot + ('db/%s/%s.html' % (number[:2], number))
-
-
 def get_package_url(system, package, mirrors=None, source=False,
                     archived=False, repeatmerged=True):
-    return (cgi_package_url(system, package, archived, source, repeatmerged) or
-            package_url(system, package, mirrors, source, repeatmerged))
+    return cgi_package_url(system, package, archived, source, repeatmerged)
 
 
-def get_report_url(system, number, mirrors=None, archived=False, mbox=False, mboxmaint=True):
-    return (cgi_report_url(system, number, archived, mbox) or
-            report_url(system, number, mirrors))
+def get_report_url(system, number, mirrors=None, archived=False, mbox=False):
+    return cgi_report_url(system, number, archived, mbox)
 
 
 def parse_bts_url(url):
@@ -1018,139 +1137,45 @@ for origin in glob.glob('/etc/dpkg/origins/*'):
         pass
 
 
-# TODO: to be removed
-def parse_html_report(number, url, http_proxy, timeout, followups=False, cgi=True):
-    page = open_url(url, http_proxy, timeout)
-    if not page:
-        return None
-
-    parser = BTSParser(cgi=cgi, followups=followups)
-    for line in page.splitlines():
-        parser.feed(line + '\n')
-    parser.close()
-
-    items = parser.preblock
-    title = "#%d: %s" % (number, parser.title)
-
-    if not followups:
-        items = [items]
-
-    output = []
-    for stuff in items:
-        parts = stuff.split('\n\n')
-        match = re.search('^Date: (.*)$', parts[0], re.M | re.I)
-        date_submitted = ''
-        if match:
-            date_submitted = 'Date: %s\n' % match.group(1)
-
-        stuff = ('\n\n'.join(parts[1:])).rstrip()
-        if not stuff:
-            continue
-
-        item = date_submitted + stuff + os.linesep
-        output.append(item)
-
-    if not output:
-        return None
-
-    return (title, output)
-
-
-# XXX: Need to handle charsets properly
-def parse_mbox_report(number, url, http_proxy, timeout, followups=False):
-    page = open_url(url, http_proxy, timeout)
-    if not page:
-        return None
-
-    # Make this seekable
-    wholefile = io.StringIO(page)
-
-    mbox = mailbox.UnixMailbox(wholefile, msgfactory)
-    title = ''
-
-    output = []
-    for message in mbox:
-        if not message:
-            pass
-
-        subject = message.get('Subject')
-        if not title:
-            title = subject
-
-        date = message.get('Date')
-        fromhdr = message.get('From')
-
-        body = entry = ''
-        for part in message.walk():
-            if part.get_content_type() == 'text/plain' and not body:
-                body = part.get_payload(None, True)
-
-        if fromhdr:
-            entry += 'From: %s%s' % (fromhdr, os.linesep)
-
-        if subject and subject != title:
-            entry += 'Subject: %s%s' % (subject, os.linesep)
-
-        if date:
-            entry += 'Date: %s%s' % (date, os.linesep)
-
-        if entry:
-            entry += os.linesep
-
-        entry += body.rstrip('\n') + os.linesep
-
-        output.append(entry)
-
-    if not output:
-        return None
-
-    title = "#%d: %s" % (number, title)
-    return (title, output)
-
-
-def get_btsroot(system, mirrors=None):
-    if mirrors:
-        alternates = SYSTEMS[system].get('mirrors')
-        for mirror in mirrors:
-            if mirror in alternates:
-                return alternates[mirror]
-    return SYSTEMS[system].get('btsroot', '')
-
-
 def get_reports(package, timeout, system='debian', mirrors=None, version=None,
                 http_proxy='', archived=False, source=False):
     if system == 'debian':
+        archived = 'both' if archived else '0'
         if isinstance(package, str):
             if source:
-                bugs = debianbts.get_bugs(src=package)
-                bugs += debianbts.get_bugs(affects='src:'+package)
+                bugs = debianbts.get_bugs(src=package, archive=archived)
+                bugs += debianbts.get_bugs(affects='src:' + package, archive=archived)
             else:
-                bugs = debianbts.get_bugs(package=package)
-                bugs += debianbts.get_bugs(affects=package)
+                bugs = debianbts.get_bugs(package=package, archive=archived)
+                bugs += debianbts.get_bugs(affects=package, archive=archived)
         else:
             bugs = []
             for pkg in package:
                 try:
-                    bugs += [ int(pkg) ]
+                    bugs += [int(pkg)]
                 except ValueError:
                     if pkg.startswith('src:'):
-                        bugs += debianbts.get_bugs(src=pkg[4:])
-                        bugs += debianbts.get_bugs(affects=pkg)
+                        bugs += debianbts.get_bugs(src=pkg[4:], archive=archived)
+                        bugs += debianbts.get_bugs(affects=pkg, archive=archived)
                     elif source:
-                        bugs += debianbts.get_bugs(src=pkg)
-                        bugs += debianbts.get_bugs(affects='src:'+pkg)
+                        bugs += debianbts.get_bugs(src=pkg, archive=archived)
+                        bugs += debianbts.get_bugs(affects='src:' + pkg, archive=archived)
                     else:
-                        bugs += debianbts.get_bugs(package=pkg)
-                        bugs += debianbts.get_bugs(affects=pkg)
+                        bugs += debianbts.get_bugs(package=pkg, archive=archived)
+                        bugs += debianbts.get_bugs(affects=pkg, archive=archived)
 
         bugs = list(set(bugs))
         # retrieve bugs and generate the hierarchy
         stats = debianbts.get_status(bugs)
 
         d = defaultdict(list)
+        r = defaultdict(list)
         for s in stats:
             # We now return debianbts.Bugreport objects, containing all the info
             # for a bug, so UIs can extract them as needed
+            if s.pending == "done":
+                r[s.severity].append(s)
+                continue
             d[s.severity].append(s)
 
         # keep the bugs ordered per severity
@@ -1162,55 +1187,13 @@ def get_reports(package, timeout, system='debian', mirrors=None, version=None,
         for sev in SEVLIST:
             if sev in d:
                 hier.append(('Bugs with severity %s' % sev, d[sev]))
+        for sev in SEVLIST:
+            if sev in r:
+                hier.append(('Resolved bugs with severity %s' % sev, r[sev]))
 
-        return (len(bugs), 'Bug reports for %s' % package, hier)
+        return (len(stats), 'Bug reports for %s' % package, hier)
 
-    # XXX: is the code below used at all now? can we remove it?
-    if isinstance(package, str):
-        if SYSTEMS[system].get('cgiroot'):
-            try:
-                result = get_cgi_reports(package, timeout, system, http_proxy, archived,
-                                         source, version=version)
-            except:
-                raise NoNetwork
-            if result:
-                return result
-
-        url = package_url(system, package, mirrors, source)
-        try:
-            page = open_url(url, http_proxy, timeout)
-        except:
-            raise NoNetwork
-        if not page:
-            return (0, None, None)
-
-        # content = page.read()
-        # if 'Maintainer' not in content:
-        #    return (0, None, None)
-
-        parser = BTSParser()
-        for line in page.splitlines():
-            parser.feed(line + '\n')
-        parser.close()
-
-        return parser.bugcount, parser.title, parser.hierarchy
-
-    # A list of bug numbers
-    this_hierarchy = []
-    package = [int(x) for x in package]
-    package.sort()
-    for bug in package:
-        result = get_report(bug, timeout, system, mirrors, http_proxy, archived)
-        if result:
-            title, body = result
-            this_hierarchy.append(title)
-            # print title
-
-    title = "Multiple bug reports"
-    bugcount = len(this_hierarchy)
-    hierarchy = [('Reports', this_hierarchy)]
-
-    return bugcount, title, hierarchy
+    raise Error(f"Getting bugs from {system} is not supported")
 
 
 def get_report(number, timeout, system='debian', mirrors=None,
@@ -1223,25 +1206,15 @@ def get_report(number, timeout, system='debian', mirrors=None,
 
         # add Date/Subject/From headers to the msg bodies
         bodies = []
-        for l in log:
-            h = l['message']
+        for lm in log:
+            h = lm['message']
             hdrs = []
             for i in ['Date', 'Subject', 'From']:
                 if i in h:
                     hdrs.append(i + ': ' + h.get(i))
-            bodies.append('\n'.join(sorted(hdrs)) + '\n\n' + l['body'])
+            bodies.append('\n'.join(sorted(hdrs)) + '\n\n' + lm['body'])
 
         # returns the bug status and a list of mail bodies
         return (status, bodies)
 
-    if SYSTEMS[system].get('cgiroot'):
-        result = get_cgi_report(number, timeout, system, http_proxy,
-                                archived, followups)
-        if result:
-            return result
-
-    url = report_url(system, number, mirrors)
-    if not url:
-        return None
-
-    return parse_html_report(number, url, http_proxy, timeout, followups, cgi=False)
+    return None
