@@ -1,7 +1,7 @@
 # hiermatch - Doing match on a list of string or a hierarchy.
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   Copyright (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2020 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
 
 import re
 from . import exceptions
@@ -16,7 +16,7 @@ def egrep_list(strlist, pattern_str, subindex=None):
 
     try:
         pat = re.compile(pattern_str, re.I | re.M)
-    except:
+    except Exception:
         raise exceptions.InvalidRegex
 
     resultlist = []
