@@ -1,7 +1,7 @@
 # reportbug_submit module - email and GnuPG functions
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   Copyright (C) 1999-2006 Chris Lawrence
-#   Copyright (C) 2008-2020 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -23,7 +23,7 @@ import sys
 import os
 import re
 import shlex
-from subprocess import Popen, STDOUT, PIPE
+from subprocess import run, Popen, STDOUT, PIPE
 import email
 import smtplib
 import socket
@@ -42,7 +42,6 @@ from .exceptions import (
 )
 from .ui import text_ui as ui
 from .utils import get_email_addr
-import errno
 
 quietly = False
 
@@ -107,6 +106,7 @@ def sign_message(body, fromaddr, package='x', pgp_addr=None, sign='gpg', draftpa
         body = None
     return body
 
+
 def _MIMEText_wrapper(text):
     msg = MIMEText(text)
     # Too long lines need to be encoded (see RFC2822), but MIMEText does
@@ -114,7 +114,7 @@ def _MIMEText_wrapper(text):
     # Since utf-8 will already be base64-encoded at this point, we only
     # need to deal with the us-ascii case.
     if msg.get_content_charset() == 'us-ascii' and \
-            max(len(l) for l in text.splitlines()) > 980:
+            max(len(ln) for ln in text.splitlines()) > 980:
         email.encoders.encode_quopri(msg)
         # due to a bug in the email library, the result now has two CTE
         # headers, only one of which is correct. Delete both and set the
@@ -123,6 +123,7 @@ def _MIMEText_wrapper(text):
         msg['Content-Transfer-Encoding'] = 'quoted-printable'
     return msg
 
+
 def mime_attach(body, attachments, charset, body_charset=None):
     mimetypes.init()
 
@@ -143,16 +144,12 @@ def mime_attach(body, attachments, charset, body_charset=None):
             failed = True
             continue
         ctype = None
-        cset = charset
         info = Popen(['file', '--mime', '--brief', '--dereference', attachment],
                      stdout=PIPE, stderr=STDOUT).communicate()[0].decode('ascii')
         if info:
             match = re.match(r'([^;, ]*)(,[^;]+)?(?:; )?(.*)', info)
             if match:
                 ctype, junk, extras = match.groups()
-                match = re.search(r'charset=([^,]+|"[^,"]+")', extras)
-                if match:
-                    cset = match.group(1)
                 # If we didn't get a real MIME type, fall back
                 if '/' not in ctype:
                     ctype = None
@@ -176,7 +173,7 @@ def mime_attach(body, attachments, charset, body_charset=None):
                 email.encoders.encode_base64(part)
         elif maintype == 'message':
             fp = open(attachment, 'rb')
-            part = MIMEMessage(email.message_from_file(fp),
+            part = MIMEMessage(email.message_from_binary_file(fp),
                                _subtype=subtype)
             fp.close()
         elif maintype == 'image':
@@ -236,7 +233,7 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
     message['To'] = sendto
 
     for (header, value) in headers:
-            message[header] = value
+        message[header] = value
 
     if ccaddr:
         message['Cc'] = ccaddr
@@ -255,9 +252,9 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
     elif mua and not (printonly or template):
         message['X-Reportbug-Version'] = VERSION_NUMBER
 
-    addrs = [str(x) for x in (message.get_all('To', []) +
-                              message.get_all('Cc', []) +
-                              message.get_all('Bcc', []))]
+    addrs = [str(x) for x in (message.get_all('To', [])
+                              + message.get_all('Cc', [])
+                              + message.get_all('Bcc', []))]
     alist = email.utils.getaddresses(addrs)
 
     cclist = [str(x) for x in message.get_all('X-Debbugs-Cc', [])]
@@ -271,20 +268,12 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
     if not outfile and not mua:
         try:
             del message['Bcc']
-        except:
+        except Exception:
             pass
 
     message = message.as_string()
     if paranoid and not (template or printonly):
-        pager = os.environ.get('PAGER', 'sensible-pager')
-        try:
-            with os.popen(pager, 'w') as p:
-                p.write(message)
-        except  Exception as e:
-            # if the PAGER exits before all the text has been sent,
-            # it'd send a SIGPIPE, so crash only if that's not the case
-            if e.errno != errno.EPIPE:
-                raise e
+        ui.display_report(message, presubj=True)
         if not ui.yes_no('Does your report seem satisfactory', 'Yes, send it.',
                          'No, don\'t send it.'):
             smtphost = mta = None
@@ -294,12 +283,8 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
         pipe = sys.stdout
     elif mua:
         pipe, filename = TempFile(prefix=tfprefix, dir=draftpath)
-    elif outfile or not ((mta and os.path.exists(mta)) and not smtphost):
-        # outfile can be None at this point
-        if outfile:
-            msgname = os.path.expanduser(outfile)
-        else:
-            msgname = '/var/tmp/%s.bug' % package
+    elif outfile:
+        msgname = os.path.expanduser(outfile)
         if os.path.exists(msgname):
             try:
                 os.rename(msgname, msgname + '~')
@@ -317,6 +302,8 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
             # we just need a place where to write() and a file handler
             # is here just for that
             pipe = fh
+    elif not ((mta and os.path.exists(mta)) or smtphost):
+        pipe, msgname = TempFile(prefix=tfprefix, dir=draftpath)
     elif (mta and os.path.exists(mta)) and not smtphost:
         try:
             x = os.getcwd()
@@ -337,8 +324,7 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
         using_sendmail = True
 
     # saving a backup of the report
-    backupfh, backupname = TempFile(prefix=tempfile_prefix(package, 'backup'), dir=draftpath)
-    ewrite('Saving a backup of the report at %s\n', backupname)
+    backupfh, backupname = TempFile(prefix=tempfile_prefix(package), dir=draftpath)
     backupfh.write(message)
     backupfh.close()
 
@@ -392,69 +378,56 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
                         tryagain = False
 
                 # In case of failure, ask to retry or to save & exit
-                if ui.yes_no('SMTP send failure: %s. Do you want to retry (or else save the report and exit)?' % x,
+                if ui.yes_no(f'SMTP send failure: {x}. You can retry, or save the report and exit. Do you want to retry?',
                              'Yes, please retry.',
                              'No, save and exit.'):
                     tryagain = True
                     continue
                 else:
                     failed = True
+                    msgname = backupname
 
-                    fh, msgname = TempFile(prefix=tfprefix, dir=draftpath)
-                    fh.write(message)
-                    fh.close()
-
-                    ewrite('Wrote bug report to %s\n', msgname)
         # Handle when some recipients are refused.
         if refused:
             for (addr, err) in refused.items():
                 ewrite('Unable to send report to %s: %d %s\n', addr, err[0],
                        err[1])
-            fh, msgname = TempFile(prefix=tfprefix, dir=draftpath)
-            fh.write(message)
-            fh.close()
-
-            ewrite('Wrote bug report to %s\n', msgname)
+            msgname = backupname
     else:
         try:
             pipe.write(message)
             pipe.flush()
-            if msgname:
-                ui.long_message("Bug report written as %s\n", msgname)
         except IOError:
             failed = True
             pipe.close()
 
         if failed or (pipe.close() and using_sendmail):
             failed = True
-            fh, msgname = TempFile(prefix=tfprefix, dir=draftpath)
-            fh.write(message)
-            fh.close()
-            ui.long_message('Error: send/write operation failed, bug report '
-                            'saved to %s\n', msgname)
+            msgname = backupname
+            ui.display_failure('Error: send/write operation failed, bug report will be saved.\n')
 
     if mua:
         ewrite("Spawning %s...\n", mua.executable)
+        sendcmd = mua.get_send_command(filename, attachments)
         returnvalue = 0
-        succeeded = False
-        while not succeeded:
-            returnvalue = ui.system(mua.get_send_command(filename, attachments))
-            if returnvalue != 0:
-                ewrite("Mutt users should be aware it is mandatory to edit the draft before sending.\n")
-                mtitle = 'Report has not been sent yet; what do you want to do now?'
-                mopts = 'Eq'
-                moptsdesc = {'e': 'Edit the message.',
-                             'q': 'Quit reportbug; will save the draft for future use.'}
-                x = ui.select_options(mtitle, mopts, moptsdesc)
-                if x == 'q':
-                    failed = True
-                    fh, msgname = TempFile(prefix=tfprefix, dir=draftpath)
-                    fh.write(message)
-                    fh.close()
-                    ui.long_message('Draft saved into %s\n', msgname)
-                    succeeded = True
+        while True:
+            if not mua.needs_terminal:
+                returnvalue = run(sendcmd, shell=True).returncode
             else:
-                succeeded = True
+                returnvalue = ui.system(sendcmd)
+            if returnvalue == 0:
+                break
+
+            ewrite("Mutt users should be aware it is mandatory to edit the draft before sending.\n")
+            mtitle = 'Report has not been sent yet; what do you want to do now?'
+            mopts = 'Eq'
+            moptsdesc = {'e': 'Edit the message.',
+                         'q': 'Quit reportbug; will save the draft for future use.'}
+            x = ui.select_options(mtitle, mopts, moptsdesc)
+            if x == 'q':
+                failed = True
+                msgname = backupname
+                break
 
     elif not failed and (using_sendmail or smtphost):
         if kudos:
@@ -477,26 +450,36 @@ def send_report(body, attachments, mua, fromaddr, sendto, ccaddr, bccaddr,
             for address in cclist:
                 ewrite('  %s\n', address)
 
-    if not (exinfo or kudos) and rtype == 'debbugs' and sysinfo and 'email' in sysinfo and not failed \
-            and mailing:
-        ewrite('\n')
-        ui.final_message(
-            """If you want to provide additional information, please wait to
-receive the bug tracking number via email; you may then send any extra
-information to %s (e.g. %s), where n is the bug number.  Normally you
-will receive an acknowledgement via email including the bug report number
-within an hour; if you haven't received a confirmation, then the bug reporting process failed at some point (reportbug or MTA failure, BTS maintenance, etc.).\n""",
-            (sysinfo['email'] % 'n'), (sysinfo['email'] % 'nnnnnn'))
-
     # If we've stored more than one copy of the message, delete the
     # one without the SMTP headers.
     if filename and os.path.exists(msgname) and os.path.exists(filename):
         try:
             os.unlink(filename)
-        except:
+        except Exception:
             pass
 
-    if filename and os.path.exists(filename) and not mua:
-        # Message is misleading if an MUA is used.
-        ewrite("A copy of the report is stored as: %s\n" % filename)
+    if backupname != msgname:
+        try:
+            os.unlink(backupname)
+        except Exception:
+            pass
+
+    if msgname and os.path.exists(msgname):
+        ui.long_message('Bug report written to a temporary file.\n'
+                        'You can resume the unsent report using:\n'
+                        f'  reportbug -r {msgname}')
+
+    elif not (exinfo or kudos) and rtype == 'debbugs' and sysinfo and 'email' in sysinfo and not failed \
+            and mailing:
+        ewrite('\n')
+        ui.final_message(
+            "If you want to provide additional information, please wait to "
+            "receive the bug tracking number via email; you may then send any extra "
+            "information to %s (e.g. %s), where n is the bug number.  Normally you "
+            "will receive an acknowledgement via email including the bug report number "
+            "within an hour; if you haven't received a confirmation, then the bug "
+            "reporting process failed at some point (reportbug or MTA failure, BTS "
+            "maintenance, etc.).\n",
+            (sysinfo['email'] % 'n'), (sysinfo['email'] % 'nnnnnn'))
+
     return
