@@ -2,7 +2,7 @@
 # tempfiles module - Temporary file handling for reportbug
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2020 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -26,16 +26,13 @@ import time
 
 
 def tempfile_prefix(package=None, extra=None):
-    if extra:
-        if package:
-            package = '%s-%s' % (package, extra)
-        else:
-            package = extra
-
+    plist = ['reportbug']
     if package:
-        return 'reportbug-%s-%s-%d-' % (
-            package, time.strftime('%Y%m%d'), os.getpid())
-    return 'reportbug-%s-%d-' % (time.strftime('%Y%m%d'), os.getpid())
+        plist.append(package)
+    if extra:
+        plist.append(extra)
+    plist.append(time.strftime('%Y%m%d%H%M%S-'))
+    return '-'.join(plist)
 
 
 template = tempfile_prefix()
@@ -62,7 +59,7 @@ def open_write_safe(filename, mode='w+b', bufsize=-1):
 
     try:
         return os.fdopen(fd, mode, bufsize)
-    except:
+    except Exception:
         os.close(fd)
         raise
 
