@@ -3,6 +3,7 @@
 #   Based on gnome-reportbug work done by Philipp Kern <pkern@debian.org>
 #   Copyright (C) 2006 Philipp Kern
 #   Copyright (C) 2008-2009 Luca Bruno
+#   Copyright (C) 2021-2022 Takahide Nojima
 #
 # This program is freely distributable per the following license:
 #
@@ -32,9 +33,6 @@ try:
     gi.require_version('GLib', '2.0')
     from gi.repository import GLib
 
-    gi.require_version('GObject', '2.0')
-    from gi.repository import GObject
-
     gi.require_version('Pango', '1.0')
     from gi.repository import Pango
 
@@ -47,20 +45,20 @@ try:
     gi.require_version('Gtk', '3.0')
     from gi.repository import Gtk
 
+    gi.require_version('GtkSource', '4')
+    from gi.repository import GtkSource
+
     gi.require_foreign('cairo')
 except ImportError:
     raise UINotImportable('Please install the reportbug-gtk package to use this interface.')
 
-global Vte
-
-gtkspellcheck = None
-
 import sys
 import re
 import traceback
 from queue import Queue
 import threading
 import textwrap
+import signal
 
 from reportbug.exceptions import NoPackage, NoBugs, QuertBTSError
 from reportbug import debbugs
@@ -70,6 +68,10 @@ ISATTY = True
 DEBIAN_LOGO = "/usr/share/pixmaps/debian-logo.png"
 
 global application, assistant, report_message, reportbug_context, ui_context
+global Vte
+
+gtkspellcheck = None
+
 
 # Utilities
 
@@ -95,9 +97,14 @@ def _assert_context(expected):
         raise AssertionError('Function should be called with %s acquired')
 
 
+def _exit():
+    os.kill(os.getpid(), signal.SIGINT)
+
+
 def highlight(s):
     return '<b>%s</b>' % s
 
+
 re_markup_free = re.compile("<.*?>")
 
 
@@ -107,7 +114,7 @@ def markup_free(s):
 
 def ask_free(s):
     s = s.strip()
-    if s[-1] in('?', ':'):
+    if s[-1] in ('?', ':'):
         return s[:-1]
     return s
 
@@ -192,12 +199,14 @@ class ExceptionDialog(CustomDialog):
     @classmethod
     def create_excepthook(cls, oldhook):
         _assert_context(reportbug_context)
+
         def excepthook(exctype, value, tb):
             # OK to call from any thread
             if oldhook:
                 oldhook(exctype, value, tb)
             application.run_once_in_main_thread(cls.start_dialog,
                                                 ''.join(traceback.format_exception(exctype, value, tb)))
+
         return excepthook
 
     @classmethod
@@ -206,8 +215,8 @@ class ExceptionDialog(CustomDialog):
         try:
             dialog = cls(tb)
             dialog.show_all()
-        except:
-            sys.exit(1)
+        except Exception:
+            _exit()
 
     def __init__(self, tb):
         _assert_context(ui_context)
@@ -228,7 +237,7 @@ class ExceptionDialog(CustomDialog):
 
     def on_response(self, dialog, res):
         _assert_context(ui_context)
-        sys.exit(1)
+        _exit()
 
 
 class ReportViewerDialog(Gtk.Dialog):
@@ -304,22 +313,31 @@ class BugReport(object):
             if not line.strip():
                 break
             self.headers.append(line)
+            if line.startswith('Subject:'):
+                break
 
         store = 0
         info = []
+        has_other = False
         while i < len(lines):
             line = lines[i]
             info.append(line)
             i += 1
-            if store < 2 and not line.strip():
-                store += 1
+            if not line.strip():
+                if store < 2:
+                    store += 1
                 continue
-            if store == 2 and(line.startswith('-- ') or line.startswith('** ')):
+            if store == 2 and (line == '-- Package-specific info:' or line == '-- System Information:'):
+                has_other = True
                 break
             store = 0
-        self.original_info = '\n'.join(info[:-3])
 
-        self.others = '\n'.join(lines[i - 1:])
+        if has_other:
+            self.original_info = '\n'.join(info[:-3])
+            self.others = '\n'.join(lines[i - 1:])
+        else:
+            self.original_info = '\n'.join(info[:-2])
+            self.others = ''
 
     def get_others(self):
         return self.others
@@ -339,33 +357,30 @@ class BugReport(object):
                 break
 
     def wrap_bug_body(self, msg, width=79, break_long_words=False):
-        """Wrap every line in the message"""
+        """Wrap every line in the message, except the pseudoheaders"""
 
         # resulting body text
         body = ''
+        phead = True
         for line in msg.splitlines():
+            if phead:
+                body += line + '\n'
+                if not line.strip():
+                    phead = False
+                continue
+
             # wrap long lines, it returns a list of "sub-lines"
             tmp = textwrap.wrap(line, width=width,
                                 break_long_words=break_long_words)
-            # need to special-case this else a join() on the list generator
-            # would remove all the '[]' so no empty lines in the report
-            if tmp == []:
-                body += '\n'
-            else:
-                # join the "sub-lines" and add a \n at the end(if there is
-                # only one item in the list, else there wouldn't be a \n)
-                body += '\n'.join(tmp) + '\n'
+            body += '\n'.join(tmp) + '\n'
 
         return body
 
     def create_message(self, info):
-        message = """%s
+        if self.others:
+            return '{}\n{}\n\n{}'.format('\n'.join(self.headers), self.wrap_bug_body(info), self.others)
 
-%s
-
-
-%s""" % ('\n'.join(self.headers), self.wrap_bug_body(info), self.others)
-        return message
+        return '{}\n{}\n'.format('\n'.join(self.headers), self.wrap_bug_body(info))
 
 
 # BTS GUI
@@ -427,7 +442,7 @@ class BugPage(Gtk.EventBox, threading.Thread):
     def pulse(self):
         _assert_context(ui_context)
         self.progress.pulse()
-        return self.isAlive()
+        return self.is_alive()
 
     def not_found(self):
         _assert_context(ui_context)
@@ -452,7 +467,10 @@ class BugPage(Gtk.EventBox, threading.Thread):
         for body in bodies:
             view = Gtk.TextView()
             view.set_editable(False)
-            view.get_buffer().set_text(body)
+            # truncate excessively long messages
+            # without the GTK interface can crash, e.g., reportbug -u gtk -N 711404 crashes
+            # TODO: fix this properly
+            view.get_buffer().set_text(body[:10000])
             if odd:
                 view.set_state_flags(Gtk.StateFlags.PRELIGHT, False)
             views.pack_start(view, False, True, 0)
@@ -652,9 +670,9 @@ class Page(ReportbugConnector):
         self.assistant.insert_page(self.widget, self.page_num)
         self.set_page_complete(self.default_complete)
         self.set_page_type(self.page_type)
+        self.set_page_title("Reportbug")
         self.assistant.set_page_side_image(self.widget, GdkPixbuf.Pixbuf.new_from_file(self.side_image))
         self.assistant.set_next_page(self)
-        self.set_page_title("Reportbug")
         # reportbug cannot usefully go back
         self.assistant.commit()
 
@@ -802,11 +820,7 @@ class GetMultilinePage(Page):
 
     def get_value(self):
         _assert_context(ui_context)
-        text = self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter())
-        lines = text.split('\n')
-        # Remove the trailing empty line at the end
-        if len(lines) > 0 and not lines[-1].strip():
-            del lines[-1]
+        text = self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter(), True)
         return text.split('\n')
 
     def execute(self, prompt):
@@ -898,6 +912,7 @@ class GetListPage(TreePage):
         _assert_context(ui_context)
         if res == Gtk.ResponseType.ACCEPT:
             self.model.append([dialog.get_value()])
+            self.validate()
         dialog.destroy()
 
     def on_remove(self, button):
@@ -909,6 +924,7 @@ class GetListPage(TreePage):
             iters.append(self.model.get_iter(path))
         for iter in iters:
             self.model.remove(iter)
+        self.validate()
 
     def execute(self, prompt):
         _assert_context(ui_context)
@@ -925,18 +941,16 @@ class GetListPage(TreePage):
         self.view.append_column(Gtk.TreeViewColumn('Item', Gtk.CellRendererText(), text=0))
 
 
-class WrapRendererText(Gtk.CellRendererText):
-    def do_render(self, cr, widget, background_area, cell_area, flags):
+class MenuPage(Page):
+    value_column = 1
+
+    def __init__(self, *args, **kwargs):
         _assert_context(ui_context)
-        self.set_property('wrap-width', cell_area.width)
-        Gtk.CellRendererText.do_render(self, cr, widget, background_area, cell_area, flags)
+        Page.__init__(self, *args, **kwargs)
 
-
-GObject.type_register(WrapRendererText)
-
-
-class MenuPage(TreePage):
-    value_column = 0
+    def setup_focus(self):
+        _assert_context(ui_context)
+        self.view.grab_focus()
 
     def create_widget(self):
         _assert_context(ui_context)
@@ -945,9 +959,25 @@ class MenuPage(TreePage):
         self.label.set_line_wrap(True)
         self.label.set_justify(Gtk.Justification.FILL)
         vbox.pack_start(self.label, False, True, 0)
-
         self.view = Gtk.TreeView()
-        self.view.set_rules_hint(True)
+        style_context = self.view.get_style_context()
+        style_context.add_class(Gtk.STYLE_CLASS_CELL)
+        # FixMe:
+        #  Replace using Gtk.StyleContext.get_background_color() with an another way,
+        # because it has been deprecated.
+        #  At this moment,one of choice is using
+        # the Gtk.StyleContext.get_property("background-color"...),but it is
+        # useless, because it has a bug shown in https://gitlab.gnome.org/GNOME/pygobject/-/issues/119
+        self.background = dict(
+            selected=self._rgba_to_string(style_context.get_background_color(Gtk.StateFlags.SELECTED)),
+            normal=self._rgba_to_string(style_context.get_background_color(Gtk.StateFlags.NORMAL)))
+        self.foreground = dict(
+            selected=self._rgba_to_string(style_context.get_color(Gtk.StateFlags.SELECTED)),
+            normal=self._rgba_to_string(style_context.get_color(Gtk.StateFlags.NORMAL)))
+        self.selection = self.view.get_selection()
+        self.selection.set_mode(Gtk.SelectionMode.NONE)
+        self.view.set_activate_on_single_click(True)
+        self.chkbox = Gtk.CellRendererToggle()
         scrolled = create_scrollable(self.view)
         scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.ALWAYS)
         vbox.pack_start(scrolled, True, True, 0)
@@ -956,25 +986,48 @@ class MenuPage(TreePage):
 
     def connect_signals(self):
         _assert_context(ui_context)
-        TreePage.connect_signals(self)
-        self.view.connect('row-activated', self.activate_forward)
+        Page.connect_signals(self)
+        self.view.connect('row-activated', self.on_toggle)
+
+    def get_value(self):
+        _assert_context(ui_context)
+        result = []
+        for row in self.model:
+            if row[0]:
+                result.append(markup_free(row[self.value_column]))
+        if len(result) > 0 and not self.multiple:
+            return result[0]
+        return result
+
+    def _rgba_to_string(self, rgba):
+        return '#{:02x}{:02x}{:02x}'.format(int(rgba.red * 255),
+                                            int(rgba.green * 255),
+                                            int(rgba.blue * 255))
+
+    def _is_suitable_radio_button(self):
+        return not self.multiple and not self.empty_ok
 
     def execute(self, par, options, prompt, default=None, any_ok=False,
                 order=None, extras=None, multiple=False):
         _assert_context(ui_context)
         GLib.idle_add(self.label.set_text, par)
-
-        self.model = Gtk.ListStore(str, str)
+        self.model = Gtk.ListStore(bool, str, str)
         self.view.set_model(self.model)
-
-        if multiple:
-            self.selection.set_mode(Gtk.SelectionMode.MULTIPLE)
-
-        self.view.append_column(Gtk.TreeViewColumn('Option', Gtk.CellRendererText(), markup=0))
-        rend = WrapRendererText()
+        self.multiple = multiple
+        self.chkbox.set_radio(self._is_suitable_radio_button())
+        column = Gtk.TreeViewColumn('Sel', self.chkbox, active=0)
+        column.set_cell_data_func(self.chkbox, self.on_cell_func)
+        self.view.append_column(column)
+        cell_text = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn('Option', cell_text, markup=1)
+        column.set_cell_data_func(cell_text, self.on_cell_func)
+        self.view.append_column(column)
+        rend = Gtk.CellRendererText()
         rend.set_property('wrap-mode', Pango.WrapMode.WORD)
-        rend.set_property('wrap-width', 300)
-        self.view.append_column(Gtk.TreeViewColumn('Description', rend, text=1))
+        rend.set_property('wrap-width', 500)
+        desc_column = Gtk.TreeViewColumn('Description', rend, text=2)
+        desc_column.set_cell_data_func(rend, self.on_cell_func)
+        self.view.append_column(desc_column)
 
         default_iter = None
         # here below, 'text' is the value of the description of the item, but
@@ -984,24 +1037,55 @@ class MenuPage(TreePage):
                 for option in order:
                     if option in options:
                         text = ' '.join(options[option].split())
-                        iter = self.model.append((highlight(option), text))
+                        iter = self.model.append([False, highlight(option), text])
                         if option == default:
                             default_iter = iter
             for option, desc in options.items():
                 if not order or option not in order:
                     text = ' '.join(desc.split())
-                    iter = self.model.append((highlight(option), text))
+                    iter = self.model.append([False, highlight(option), text])
                     if option == default:
                         default_iter = iter
         else:
             for row in options:
                 text = ' '.join(row[1].split())
-                iter = self.model.append((highlight(row[0]), text))
+                iter = self.model.append([False, highlight(row[0]), text])
                 if row[0] == default:
                     default_iter = iter
 
         if default_iter:
-            self.selection.select_iter(default_iter)
+            self.model.set_value(default_iter, 0, True)
+
+        self.validate()
+
+    def on_toggle(self, widget, path, data):
+        if not self.multiple:
+            selected_path = Gtk.TreePath(path)
+            for row in self.model:
+                if row.path == selected_path:
+                    if not self.empty_ok:
+                        row[0] = True
+                    else:
+                        row[0] = not row[0]
+                else:
+                    row[0] = False
+        else:
+            self.model[path][0] = not self.model[path][0]
+        self.validate()
+
+    def on_cell_func(self, column, cell, model, it, data):
+        if model.get_value(it, 0):
+            if isinstance(cell, Gtk.CellRendererToggle):
+                cell.set_property("cell-background", self.background['selected'])
+            else:
+                cell.set_property("background", self.background['selected'])
+                cell.set_property("foreground", self.foreground['selected'])
+        else:
+            if isinstance(cell, Gtk.CellRendererToggle):
+                cell.set_property("cell-background", self.background['normal'])
+            else:
+                cell.set_property("background", self.background['normal'])
+                cell.set_property("foreground", self.foreground['normal'])
 
 
 class HandleBTSQueryPage(TreePage):
@@ -1061,12 +1145,7 @@ class HandleBTSQueryPage(TreePage):
 
                 report = []
                 for category, bugs in hierarchy:
-                    buglist = []
-                    for bug in bugs:
-                        buglist.append(bug)
-                    # XXX: this needs to be fixed in debianbts; Bugreport are
-                    # not sortable(on bug_num) - see #639458
-                    sorted(buglist, reverse=latest_first)
+                    buglist = sorted(bugs, key=lambda b: b.bug_num, reverse=latest_first)
                     report.append((category, list(map(Bug, buglist))))
 
                 return(report, sectitle), {}
@@ -1271,23 +1350,21 @@ class LongMessagePage(Page):
     def execute(self, message, *args):
         _assert_context(ui_context)
         message = message % args
-        # make it all on one line, it will be wrapped at display-time
-        message = ' '.join(message.split())
-        GLib.idle_add(self.label.set_text, message)
-        # Reportbug should use final_message, so emulate it
+        # no need to wrap the message, it will be wrapped at display-time
+        # but separate all paragraphs by an empty line
+        message = '\n\n'.join(par for par in message.splitlines() if par)
         if('nnnnnn' in message):
-            self.set_page_type(Gtk.AssistantPageType.CONFIRM)
-            self.set_page_title("Thanks for your report")
+            message = 'Thank you for your report.\n\n' + message
+        GLib.idle_add(self.label.set_text, message)
 
 
 class FinalMessagePage(LongMessagePage):
-    page_type = Gtk.AssistantPageType.CONFIRM
+    page_type = Gtk.AssistantPageType.SUMMARY
     default_complete = True
 
     def execute(self, *args, **kwargs):
         _assert_context(ui_context)
         LongMessagePage.execute(self, *args, **kwargs)
-        #self.set_page_title("Thanks for your report")
 
 
 class EditorPage(Page):
@@ -1300,7 +1377,8 @@ class EditorPage(Page):
         hbox.pack_start(self.subject, True, True, 0)
         vbox.pack_start(hbox, False, True, 0)
 
-        self.view = Gtk.TextView()
+        self.info_buffer = GtkSource.Buffer()
+        self.view = GtkSource.View(buffer=self.info_buffer)
         self.view.modify_font(Pango.FontDescription("Monospace"))
         self.view.set_wrap_mode(Gtk.WrapMode.WORD)
 
@@ -1311,17 +1389,20 @@ class EditorPage(Page):
         if gtkspellcheck is None:
             try:
                 import gtkspellcheck
-            except:
+            except ImportError:
                 gtkspellcheck = NotImplemented
 
         if gtkspellcheck is not NotImplemented:
-            gtkspellcheck.SpellChecker(self.view)
-        self.info_buffer = self.view.get_buffer()
+            try:
+                gtkspellcheck.SpellChecker(self.view)
+            except Exception:
+                pass
         scrolled = create_scrollable(self.view)
         vbox.pack_start(scrolled, True, True, 0)
 
         expander = Gtk.Expander.new_with_mnemonic("Other system information")
         view = Gtk.TextView()
+        view.modify_font(Pango.FontDescription("Monospace"))
         view.set_editable(False)
         self.others_buffer = view.get_buffer()
         scrolled = create_scrollable(view)
@@ -1388,7 +1469,8 @@ class EditorPage(Page):
         self.others_buffer.set_text(self.report.get_others())
 
         info = self.report.get_original_info()
-        if info.strip() == "*** Please type your report below this line ***":
+        # if info.strip() == "*** Please type your report below this line ***":
+        if info.strip() == "Dear Maintainer,":
             info = "Please type your report here.\nThe text will be wrapped to be max 79 chars long per line."
             self.handle_first_info()
         self.info_buffer.set_text(info)
@@ -1489,7 +1571,7 @@ class SystemPage(Page):
 
     def execute(self, cmdline):
         _assert_context(ui_context)
-        self.terminal.spawn_sync(Vte.PtyFlags.DEFAULT, os.environ['HOME'], ['/bin/bash', '-c', cmdline], [], GLib.SpawnFlags.DO_NOT_REAP_CHILD, None, None)
+        self.terminal.spawn_async(Vte.PtyFlags.DEFAULT, os.environ['HOME'], ['/bin/bash', '-c', cmdline], None, GLib.SpawnFlags.DEFAULT, None, None, -1, None)
 
 
 class ProgressPage(Page):
@@ -1575,6 +1657,7 @@ class ReportbugAssistant(Gtk.Assistant):
         self.connect('prepare', self.on_prepare)
         self.connect('delete-event', self.close)
         self.connect('apply', self.close)
+        self.connect('close', self.close)
 
     def on_back_show(self, widget):
         _assert_context(ui_context)
@@ -1599,7 +1682,7 @@ class ReportbugAssistant(Gtk.Assistant):
 
     def close(self, *args):
         _assert_context(ui_context)
-        sys.exit(0)
+        _exit()
 
     def confirm_exit(self, *args):
         _assert_context(ui_context)
@@ -1609,7 +1692,7 @@ class ReportbugAssistant(Gtk.Assistant):
         response = dialog.run()
         dialog.destroy()
         if response == Gtk.ResponseType.YES:
-            sys.exit(0)
+            _exit()
 
     def forward(self, page_num):
         _assert_context(ui_context)
@@ -1718,16 +1801,17 @@ def select_multiple(*args, **kwargs):
     _assert_context(reportbug_context)
     kwargs['multiple'] = True
     kwargs['empty_ok'] = True
-    return menu(*args, **kwargs)
+    return menu(*args, **kwargs)  # noqa: F821
 
 
 def get_multiline(prompt, *args, **kwargs):
     _assert_context(reportbug_context)
     if 'ENTER' in prompt:
         # This is a list, let's handle it the best way
-        return get_list(prompt, *args, **kwargs)
+        return get_list(prompt, *args, **kwargs)  # noqa: F821
     else:
-        return _get_multiline(prompt, *args, **kwargs)
+        return _get_multiline(prompt, *args, **kwargs)  # noqa: F821
+
 
 pages = {'get_string': GetStringPage,
          'get_password': GetPasswordPage,
@@ -1751,6 +1835,7 @@ dialogs = {'yes_no': YesNoDialog,
 
 def create_forwarder(parent, klass):
     _assert_context(reportbug_context)
+
     def func(*args, **kwargs):
         _assert_context(reportbug_context)
         op = application.call_in_main_thread(klass, parent)
@@ -1760,6 +1845,7 @@ def create_forwarder(parent, klass):
             return e.result
         application.run_once_in_main_thread(op.execute_operation, *args, **kwargs)
         return application.get_last_value()
+
     return func
 
 
@@ -1775,7 +1861,7 @@ def initialize():
     try:
         gi.require_version('Vte', '2.91')
         from gi.repository import Vte
-    except (ImportError,ValueError):
+    except (ImportError, ValueError):
         message = """Please install the %s package to use the GTK+ (known as 'gtk' in reportbug) interface.
 Falling back to 'text' interface."""
         dialog = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT,
