@@ -1,7 +1,7 @@
 # Text user interface for reportbug
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   (C) 2001-08 Chris Lawrence
-#   Copyright (C) 2008-2020 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -38,7 +38,7 @@ except ImportError:
 
 from reportbug import debbugs, hiermatch
 from reportbug.exceptions import (
-    NoReport, NoPackage, NoBugs, NoNetwork, QuertBTSError,
+    NoReport, NoPackage, NoBugs, QuertBTSError,
     InvalidRegex,
 )
 from reportbug.urlutils import launch_browser
@@ -50,7 +50,7 @@ charset = 'us-ascii'
 try:
     r, c = subprocess.getoutput('stty size').split()
     rows, columns = int(r) or 24, int(c) or 79
-except:
+except Exception:
     rows, columns = 24, 79
 
 
@@ -64,13 +64,14 @@ def ewrite(message, *args):
     sys.stderr.write(message)
     sys.stderr.flush()
 
+
 log_message = ewrite
 display_failure = ewrite
 
 
 def system(cmdline):
     try:
-        x = os.getcwd()
+        _ = os.getcwd()
     except OSError:
         os.chdir('/')
     return os.system(cmdline)
@@ -96,24 +97,25 @@ def indent_wrap_text(text, starttext='', indent=0, linelen=None):
         return output
     return output + '\n'
 
+
 # Readline support, if available
 if readline is not None:
     readline.parse_and_bind("tab: complete")
     try:
         # minimize the word delimiter list if possible
         readline.set_completer_delims(' ')
-    except:
+    except Exception:
         pass
 
 
 def _launch_mbox_reader(mbox_reader_cmd, bts, bugs, number, mirrors, archived,
-                        mbox, mboxmaint, http_proxy, timeout):
+                        mbox, http_proxy, timeout):
     try:
         number = int(number)
         if number not in bugs and 1 <= number <= len(bugs):
             number = bugs[number - 1]
         reportbug.utils.launch_mbox_reader(mbox_reader_cmd,
-                                           debbugs.get_report_url(bts, number, mirrors, archived, mbox, mboxmaint),
+                                           debbugs.get_report_url(bts, number, mirrors, archived, mbox),
                                            http_proxy, timeout)
     except ValueError:
         ewrite('Invalid report number: %s\n',
@@ -175,6 +177,10 @@ def select_options(msg, ok, help, allow_numbers=None, nowrap=False):
     if '?' not in ok:
         ok = ok + '?'
 
+    msg = msg.rstrip()
+    if msg[-1] == '?':
+        msg = msg[:-1]
+
     if nowrap:
         longmsg = msg + ' [' + '|'.join(ok) + ']?' + ' '
     else:
@@ -242,9 +248,16 @@ def yes_no(msg, yeshelp, nohelp, default=True, nowrap=False):
 
 def long_message(text, *args):
     if args:
-        ewrite(indent_wrap_text(text % args))
+        mtext = text % args
     else:
-        ewrite(indent_wrap_text(text))
+        mtext = text
+
+    linelen = columns - 1
+    # we do not use indent_wrap_text here because it does not preserve newlines
+    mtext = '\n'.join(textwrap.fill(par, width=linelen) for par in mtext.splitlines())
+    mtext = mtext.rstrip() + '\n'
+    ewrite(mtext)
+
 
 final_message = long_message
 
@@ -271,14 +284,14 @@ def get_string(prompt, options=None, title=None, empty_ok=False, force_prompt=Fa
 def get_multiline(prompt):
     ewrite('\n')
     ewrite(indent_wrap_text(prompt + "  Press ENTER on a blank line to continue.\n"))
-    l = []
+    ret = []
     while 1:
         entry = get_string('', force_prompt=True).strip()
         if not entry:
             break
-        l.append(entry)
+        ret.append(entry)
     ewrite('\n')
-    return l
+    return ret
 
 
 def get_password(prompt=None):
@@ -392,7 +405,10 @@ def menu(par, options, prompt, default=None, title=None, any_ok=False,
                 if response == 'done':
                     return list(selected.keys())
                 elif response == 'none':
-                    return []
+                    if len(selected) == 0:
+                        return []
+                    else:
+                        selected = {}
                 elif selected.get(response):
                     del selected[response]
                 else:
@@ -426,7 +442,7 @@ def show_report(number, system, mirrors,
     try:
         info = debbugs.get_report(number, timeout, system, mirrors=mirrors,
                                   followups=1, http_proxy=http_proxy, archived=archived)
-    except:
+    except Exception:
         info = None
 
     if not info:
@@ -488,7 +504,7 @@ def show_report(number, system, mirrors,
             skip_pager = True
         elif x == 'e':
             reportbug.utils.launch_mbox_reader(mbox_reader_cmd,
-                                               debbugs.get_report_url(system, number, mirrors, archived, True, True),
+                                               debbugs.get_report_url(system, number, mirrors, archived, True),
                                                http_proxy, timeout)
             skip_pager = True
         elif x == 'o':
@@ -560,23 +576,20 @@ def handle_bts_query(package, bts, timeout, mirrors=None, http_proxy="",
             # first item is the title of the section
             entry_new = entry[0]
             bugs_new = {}
-            bugs_numbers = []
-            # XXX: we can probably simplify this code, with some lists
-            # generations and map()
-            # second item is a list of bugs report
+            # second item is a list of bug report objects
             for bug in entry[1]:
-                # show if the bugs is already resolved
+                # show if the bug is already resolved
                 done = ''
                 if bug.pending == 'done':
                     done = '  [RESOLVED]'
                 # we take the info we need (bug number and subject)
-                # we use a dict so it's easy to sort the keys and then values
+                # we use a dict so it's easy to sort the keys
                 bugs_new[bug.bug_num] = "%s%s" % (bug.subject, done)
-                # and at the same time create a list of bugs numbers
-                bugs_numbers.append(bug.bug_num)
-            # then we sort both the lists
-            hierarchy_new.append((entry_new, ["#%d  %s" % (k, bugs_new[k]) for k in sorted(list(bugs_new.keys()), reverse=latest_first)]))
-            bugs.extend(sorted(bugs_numbers, reverse=latest_first))
+            # then we sort the list
+            bugs_numbers = sorted(bugs_new.keys(), reverse=latest_first)
+            hierarchy_new.append((entry_new, ["{:>8}  {}".format(f"#{k}", bugs_new[k])
+                                              for k in bugs_numbers]))
+            bugs.extend(bugs_numbers)
 
         # replace old hierarchy with hierarchy_new
         hierarchy = hierarchy_new
@@ -604,7 +617,7 @@ def browse_bugs(hierarchy, count, bugs, bts, queryonly, mirrors,
                 http_proxy, timeout, screen, title, package,
                 source, mbox_reader_cmd):
     try:
-        output_encoding = locale.getpreferredencoding()
+        _ = locale.getpreferredencoding()
     except locale.Error as msg:
         print(msg)
         sys.exit(1)
@@ -619,16 +632,13 @@ def browse_bugs(hierarchy, count, bugs, bts, queryonly, mirrors,
         catname, reports = category[0:2]
         while catname.endswith(':'):
             catname = catname[:-1]
-        total = len(reports)
 
         while len(reports):
             these = reports[:rows - 2]
             reports = reports[rows - 2:]
             remain = len(reports)
 
-            tplural = rplural = 's'
-            if total == 1:
-                tplural = ''
+            rplural = 's'
             if remain != 1:
                 rplural = ''
 
@@ -638,7 +648,6 @@ def browse_bugs(hierarchy, count, bugs, bts, queryonly, mirrors,
             else:
                 lastpage.append(catname + '\n')
 
-            oldscount, oldecount = scount, endcount
             for report in these:
                 scount = scount + 1
                 endcount = endcount + 1
@@ -728,7 +737,7 @@ def browse_bugs(hierarchy, count, bugs, bts, queryonly, mirrors,
                     elif x == 'e':
                         number = our_raw_input('Please enter the number of the bug you would like to view: #', allowed)
                         _launch_mbox_reader(mbox_reader_cmd, bts, bugs, number,
-                                            mirrors, 'no', True, True, http_proxy,
+                                            mirrors, 'no', True, http_proxy,
                                             timeout)
                     else:
                         if x == 'm' or x == 'i':
@@ -786,7 +795,7 @@ def proc_hierarchy(hierarchy):
 
     # Copy & paste from handle_bts_query()
     bugs = []
-    exp = re.compile(r'\#(\d+)[ :]')
+    exp = re.compile(r'\ *\#(\d+)[ :]')
     for entry in hierarchy or []:
         for bug in entry[1]:
             match = exp.match(bug)
@@ -801,7 +810,7 @@ def search_bugs(hierarchyfull, bts, queryonly, mirrors,
     """Return string "FilterEnd" when we are done with search."""
 
     try:
-        output_encoding = locale.getpreferredencoding()
+        _ = locale.getpreferredencoding()
     except locale.Error as msg:
         print(msg)
         sys.exit(1)
@@ -819,7 +828,6 @@ def search_bugs(hierarchyfull, bts, queryonly, mirrors,
         return "FilterEnd"
 
     count, bugs = proc_hierarchy(hierarchy)
-    exp = re.compile(r'\#(\d+):')
 
     if not count:
         our_raw_input('No match found, press ENTER to continue.')
@@ -858,7 +866,6 @@ def search_bugs(hierarchyfull, bts, queryonly, mirrors,
                 lastpage.append(' %s: %d report%s\n' %
                                 (catname, total, tplural))
 
-            oldscount, oldecount = scount, endcount
             for report in these:
                 scount = scount + 1
                 endcount = endcount + 1
@@ -953,7 +960,7 @@ def search_bugs(hierarchyfull, bts, queryonly, mirrors,
                         number = our_raw_input('Please enter the number of the '
                                                'bug you would like to view: #', allowed)
                         _launch_mbox_reader(mbox_reader_cmd, bts, bugs, number,
-                                            mirrors, 'no', True, True, http_proxy, timeout)
+                                            mirrors, 'no', True, http_proxy, timeout)
                     else:
                         if x == 'm' or x == 'i':
                             number = our_raw_input(
@@ -1034,8 +1041,8 @@ def spawn_editor(message, filename, editor, charset='utf-8'):
     if ourline:
         if edname in ('vi', 'nvi', 'vim', 'elvis', 'gvim', 'kvim'):
             opts = '-c :%d' % ourline
-        elif (edname in ('elvis-tiny', 'gnuclient', 'ee', 'pico', 'nano', 'zile') or
-              'emacs' in edname):
+        elif (edname in ('elvis-tiny', 'gnuclient', 'ee', 'pico', 'nano', 'zile')
+              or 'emacs' in edname):
             opts = '+%d' % ourline
         elif edname in ('jed', 'xjed'):
             opts = '-g %d' % ourline
