@@ -2,7 +2,7 @@
 # utils module - common functions for reportbug UIs
 #   Written by Chris Lawrence <lawrencc@debian.org>
 #   Copyright (C) 1999-2008 Chris Lawrence
-#   Copyright (C) 2008-2020 Sandro Tosi <morph@debian.org>
+#   Copyright (C) 2008-2022 Sandro Tosi <morph@debian.org>
 #
 # This program is freely distributable per the following license:
 #
@@ -41,22 +41,16 @@ import socket
 import subprocess
 import apt
 import gzip
-import urllib
 
 from .urlutils import open_url
 from .mailer import MUA
 
-# Paths for dpkg
-DPKGLIB = '/var/lib/dpkg'
-AVAILDB = os.path.join(DPKGLIB, 'available')
-STATUSDB = os.path.join(DPKGLIB, 'status')
-
 # Headers other than these become email headers for debbugs servers
 PSEUDOHEADERS = ('Package', 'Source', 'Version', 'Severity', 'File', 'Tags',
                  'Justification', 'Followup-For', 'Owner', 'User', 'Usertags',
                  'Forwarded', 'Control', 'X-Debbugs-Cc')
 # These pseudo-headers can be repeated in the report
-REPEATABLE_PSEUDOHEADERS = ['Control',]
+REPEATABLE_PSEUDOHEADERS = ['Control', ]
 
 MODES = {'novice': 'Offer simple prompts, bypassing technical questions.',
          'standard': 'Offer more extensive prompts, including asking about '
@@ -73,11 +67,8 @@ for mode in MODELIST:
 del mode
 
 # moved here since it needs the MODE_* vars to be defined
-from . import debbugs
-# it needs to be imported after debbugs
-#from .ui import text_ui as ui
-
-from reportbug.ui import AVAILABLE_UIS
+from . import debbugs   # noqa: E402
+from reportbug.ui import AVAILABLE_UIS   # noqa: E402
 
 NEWBIELINE = """Dear Maintainer,
 
@@ -95,18 +86,20 @@ fhs_directories = ['/', '/usr', '/usr/share', '/var', '/usr/X11R6',
                    '/usr/man', '/usr/doc', '/usr/bin']
 
 # A map between codenames and suites
-CODENAME2SUITE = {'wheezy': 'oldoldoldstable',
-                  'jessie': 'oldoldstable',
-                  'stretch': 'oldstable',
-                  'buster': 'stable',
-                  'bullseye': 'testing',
-                  'bookworm': 'next-testing',
+CODENAME2SUITE = {'wheezy': 'oldoldoldoldstable',
+                  'jessie': 'oldoldoldstable',
+                  'stretch': 'oldoldstable',
+                  'buster': 'oldstable',
+                  'bullseye': 'stable',
+                  'bookworm': 'testing',
+                  'trixie': 'next-testing',
                   'sid': 'unstable',
                   'experimental': 'experimental'}
 SUITE2CODENAME = dict([(suite, codename) for codename, suite in list(CODENAME2SUITE.items())])
 
 _apt_cache = apt.Cache()
 
+
 def realpath(filename):
     filename = os.path.abspath(filename)
 
@@ -194,7 +187,7 @@ def get_command_output(cmd):
 
 def query_dpkg_for(filename, use_dlocate=True):
     try:
-        x = os.getcwd()
+        _ = os.getcwd()
     except OSError:
         os.chdir('/')
     searchfilename = glob_escape(filename)
@@ -255,23 +248,25 @@ def find_package_for(filename, pathonly=False):
 
 
 def find_rewritten(username):
-    for filename in ['/etc/email-addresses']:
-        if os.path.exists(filename):
-            try:
-                fp = open(filename, errors='backslashreplace')
-            except IOError:
+    filename = '/etc/email-addresses'
+    try:
+        fp = open(filename, errors='backslashreplace')
+    except (FileNotFoundError, IOError):
+        return None
+    try:
+        for line in fp:
+            line = line.strip().split('#')[0]
+            if not line:
                 continue
-            for line in fp:
-                line = line.strip().split('#')[0]
-                if not line:
-                    continue
-                try:
-                    name, alias = line.split(':')
-                    if name.strip() == username:
-                        return alias.strip()
-                except ValueError:
-                    print('Invalid entry in %s' % filename)
-                    return None
+            try:
+                name, alias = line.split(':')
+                if name.strip() == username:
+                    return alias.strip()
+            except ValueError:
+                print('Invalid entry in %s' % filename)
+                return None
+    finally:
+        fp.close()
 
 
 def check_email_addr(addr):
@@ -287,11 +282,18 @@ def check_email_addr(addr):
         return False
     if domainpart.startswith('.') or domainpart.endswith('.'):
         return False
+    # known invalid addresses according to rfc2606
+    if domainpart in ('localhost', 'example.com', 'example.net', 'example.org'):
+        return False
+    if domainpart.endswith(('.example', '.invalid', '.localhost', '.test',
+                            '.example.com', '.example.net', '.example.org')):
+        return False
+
     return True
 
 
 def get_email_addr(addr):
-    return email.utils.getaddresses([addr,])[0]
+    return email.utils.getaddresses([addr, ])[0]
 
 
 def get_email(emailaddr='', realname=''):
@@ -301,15 +303,17 @@ def get_email(emailaddr='', realname=''):
 def get_user_id(emailaddr='', realname='', charset='utf-8'):
     uid = os.getuid()
     info = pwd.getpwuid(uid)
-    emailaddr = (os.environ.get('REPORTBUGEMAIL', emailaddr) or
-                 os.environ.get('DEBEMAIL') or os.environ.get('EMAIL'))
+    emailaddr = (os.environ.get('REPORTBUGEMAIL', emailaddr)
+                 or os.environ.get('DEBEMAIL')
+                 or os.environ.get('EMAIL'))
 
     emailaddr = emailaddr or find_rewritten(info[0]) or info[0]
 
     if '@' not in emailaddr:
-        if os.path.exists('/etc/mailname'):
-            domainname = open('/etc/mailname').readline().strip()
-        else:
+        try:
+            with open('/etc/mailname', 'r') as mf:
+                domainname = mf.readline().strip()
+        except (FileNotFoundError, IOError):
             domainname = socket.getfqdn()
 
         emailaddr = emailaddr + '@' + domainname
@@ -351,7 +355,6 @@ def get_package_status(package, avail=False):
     originre = re.compile('Origin: ')
     bugsre = re.compile('Bugs: ')
     descre = re.compile('Description(?:-[a-zA-Z]+)?: ')
-    fullre = re.compile(' ')
     srcre = re.compile('Source: ')
     sectionre = re.compile('Section: ')
 
@@ -367,7 +370,7 @@ def get_package_status(package, avail=False):
     state = ''
 
     try:
-        x = os.getcwd()
+        _ = os.getcwd()
     except OSError:
         os.chdir('/')
 
@@ -416,7 +419,7 @@ def get_package_status(package, avail=False):
             (crud, origin) = line.split(": ", 1)
         elif bugsre.match(line):
             (crud, bugs) = line.split(": ", 1)
-        elif descre.match(line):
+        elif descre.match(line) and not fulldesc:
             (crud, desc) = line.split(": ", 1)
             descmode = True
         elif dependsre.match(line):
@@ -527,13 +530,8 @@ class AvailDB(object):
 
 
 def get_dpkg_database():
-    try:
-        fp = open(STATUSDB, errors="backslashreplace")
-        if fp:
-            return AvailDB(fp=fp)
-    except IOError:
-        print('Unable to open', STATUSDB, file=sys.stderr)
-        sys.exit(1)
+    subp = subprocess.Popen(('dpkg-query', '--status'), errors="backslashreplace", stdout=subprocess.PIPE)
+    return AvailDB(popenob=subp)
 
 
 def get_avail_database():
@@ -548,36 +546,55 @@ def get_source_name(package):
     except KeyError:
         pass
     # check if there is a source package with that name
-    if apt.apt_pkg.SourceRecords().lookup(package):
-        return package
+    try:
+        srcrecords = apt.apt_pkg.SourceRecords()
+        if srcrecords.lookup(package):
+            return srcrecords.package
+    except apt.apt_pkg.Error as e:
+        print(f"Cannot look up source package: '{e}'")
     return None
 
 
-def get_source_package(package):
+def get_source_version(srcname):
+    try:
+        srcrecords = apt.apt_pkg.SourceRecords()
+        while srcrecords.lookup(srcname):
+            if srcrecords.package == srcname:
+                return srcrecords.version
+    except apt.apt_pkg.Error as e:
+        print(f"Cannot look up source package: '{e}'")
+    return None
+
+
+def get_source_package(package, only_source=False):
     packages = []
-    retlist = []
-    found = {}
+    found = set()
+    try:
+        srcrecords = apt.apt_pkg.SourceRecords()
+    except apt.apt_pkg.Error as e:
+        print(f"Cannot look up source package: '{e}'")
+        return packages
 
-    data = get_command_output('apt-cache showsrc ' + shlex.quote(package))
-    binre = re.compile(r'^Binary: (.*)$')
-    for line in data.split('\n'):
-        m = binre.match(line)
-        if m:
-            packs = m.group(1)
-            packlist = re.split(r',\s*', packs)
-            packages += packlist
-
-    for p in packages:
-        try:
-            desc = _apt_cache[p].versions[0].summary
-        except KeyError:
+    while srcrecords.lookup(package):
+        if srcrecords.package in found:
             continue
-        if desc and (p not in found):
-            retlist += [(p, desc)]
-            found[p] = desc
 
-    retlist.sort()
-    return retlist
+        if only_source and srcrecords.package != package:
+            continue
+
+        found.add(srcrecords.package)
+
+        for bp in sorted(srcrecords.binaries):
+            try:
+                desc = _apt_cache[bp].versions[0].summary
+            except KeyError:
+                continue
+            if desc:
+                packages += [(bp, desc)]
+
+        packages += [('src:' + srcrecords.package, 'Source package')]
+
+    return packages
 
 
 def get_package_info(packages, skip_notfound=False):
@@ -599,8 +616,8 @@ def get_package_info(packages, skip_notfound=False):
         # Package regular expression
         r'^(?P<hdr>Package):\s+(' + '|'.join(searchpkgs) + ')$',
         # Provides regular expression
-        r'^(?P<hdr>Provides):\s+' + pkgname + r'*(?P<pkg>' + '|'.join(searchpkgs) +
-        r')(?:$|,\s+)' + pkgname + '*$'
+        r'^(?P<hdr>Provides):\s+' + pkgname + r'*(?P<pkg>'
+        + '|'.join(searchpkgs) + r')(?:$|,\s+)' + pkgname + '*$'
     ]
 
     groups = list(groupfor.values())
@@ -831,13 +848,13 @@ def get_multiarch():
 def generate_blank_report(package, pkgversion, severity, justification,
                           depinfo, confinfo, foundfile='', incfiles='',
                           system='debian', exinfo=None, type=None, klass='',
-                          subject='', tags='', body='', mode=MODE_EXPERT,
+                          subject='', tags='', body='', mode=MODE_EXPERT,  # noqa: F821
                           pseudos=None, debsumsoutput=None, issource=False,
                           options=None):
     # For now...
     from . import bugreport
 
-    sysinfo = (package not in ('wnpp', 'ftp.debian.org') and (options and not options.buildd_format))
+    sysinfo = (package not in debbugs.debother and (options and not options.buildd_format))
 
     # followup is where bugreport expects the notification of the bug reportbug
     # to follow-up, but reportbug pass this information with 'exinfo'
@@ -870,7 +887,8 @@ CONFIG_ARGS = (
     'sign', 'nocc', 'nocompress', 'dontquery', 'noconf', 'mirrors', 'keyid',
     'headers', 'interface', 'template', 'mode', 'check_available', 'query_src',
     'printonly', 'offline', 'check_uid', 'smtptls', 'smtpuser', 'smtppasswd',
-    'paranoid', 'mbox_reader_cmd', 'max_attachment_size', 'listccme', 'outfile')
+    'paranoid', 'mbox_reader_cmd', 'max_attachment_size', 'listccme',
+    'outfile', 'draftpath', 'timeout')
 
 
 def first_run():
@@ -883,7 +901,7 @@ def parse_config_files():
         if os.path.exists(filename):
             try:
                 lex = our_lex(open(filename, errors="backslashreplace"), posix=True)
-            except IOError as msg:
+            except IOError:
                 continue
 
             lex.wordchars = lex.wordchars + '-.@/:<>'
@@ -914,7 +932,7 @@ def parse_config_files():
                 elif token in ('email', 'realname', 'replyto', 'http_proxy',
                                'smtphost', 'editor', 'mua', 'mta', 'smtpuser',
                                'smtppasswd', 'justification', 'keyid',
-                               'mbox_reader_cmd', 'outfile'):
+                               'mbox_reader_cmd', 'outfile', 'draftpath'):
                     bit = lex.get_token()
                     args[token] = bit
                 elif token in ('no-smtptls', 'smtptls'):
@@ -948,7 +966,7 @@ def parse_config_files():
                 elif token == 'reportbug_version':
                     # Currently ignored; might be used for compat purposes
                     # eventually
-                    w_version = lex.get_token().lower()
+                    _ = lex.get_token().lower()
                 elif token in MUA:
                     args['mua'] = MUA[token]
                 elif token in ('query-source', 'no-query-source'):
@@ -964,6 +982,9 @@ def parse_config_files():
                 elif token == 'max_attachment_size':
                     arg = lex.get_token()
                     args['max_attachment_size'] = int(arg)
+                elif token == 'timeout':
+                    arg = lex.get_token()
+                    args['timeout'] = int(arg)
                 elif token == 'envelopefrom':
                     token = lex.get_token().lower()
                     args['envelopefrom'] = token
@@ -1038,9 +1059,9 @@ def cleanup_msg(dmessage, headers, pseudos, btstype):
             mob = headerre.match(line)
             # GNATS and debbugs have different ideas of what a pseudoheader
             # is...
-            if mob and ((btstype == 'debbugs' and
-                                 normph(mob.group(1)) not in accepted_pseudoheaders) or
-                        (btstype == 'gnats' and mob.group(1)[0] != '>')):
+            if mob and ((btstype == 'debbugs'
+                         and normph(mob.group(1)) not in accepted_pseudoheaders)
+                        or (btstype == 'gnats' and mob.group(1)[0] != '>')):
                 # unrecognized pseudoheaders are turned into headers
                 newheaders.append(mob.groups())
                 lastpseudo = False
@@ -1148,7 +1169,7 @@ def exec_and_parse_bugscript(handler, bugscript, runner=os.system):
     fh, filename = TempFile()
     fh.close()
     rc = runner('LC_ALL=C %s %s %s' % (handler, shlex.quote(bugscript),
-                                          shlex.quote(filename)))
+                                       shlex.quote(filename)))
 
     isheaders = False
     ispseudoheaders = False
@@ -1203,15 +1224,23 @@ def get_init_system():
 
     if os.path.isdir('/run/systemd/system'):
         init = 'systemd (via /run/systemd/system)'
-    if not subprocess.call('. /lib/lsb/init-functions ; init_is_upstart', shell=True):
+    elif not subprocess.call('. /lib/lsb/init-functions ; init_is_upstart', shell=True):
         init = 'upstart (via init_is_upstart())'
     elif os.path.isfile('/run/runit.stopit'):
         init = 'runit (via /run/runit.stopit)'
+    elif os.path.isdir('/run/openrc'):
+        init = 'OpenRC (via /run/openrc)'
+        try:
+            with open('/proc/1/comm', 'r') as pf:
+                init += f', PID 1: {pf.read().strip()}'
+        except Exception:
+            pass
     elif os.path.isfile('/sbin/init') and not os.path.islink('/sbin/init'):
         init = 'sysvinit (via /sbin/init)'
 
     return init
 
+
 def get_lsm_info():
     """Determines the linux security module enabled on the current machine
 
@@ -1246,24 +1275,25 @@ def get_kernel_taint_flags():
 
     # https://github.com/torvalds/linux/blob/cedc5b6aab493f6b1b1d381dccc0cc082da7d3d8/include/linux/kernel.h#L582
     # this is going to need updating (but maybe not that often)
-    TAINT_FLAGS = ['TAINT_PROPRIETARY_MODULE',
-                   'TAINT_FORCED_MODULE',
-                   'TAINT_CPU_OUT_OF_SPEC',
-                   'TAINT_FORCED_RMMOD',
-                   'TAINT_MACHINE_CHECK',
-                   'TAINT_BAD_PAGE',
-                   'TAINT_USER',
-                   'TAINT_DIE',
-                   'TAINT_OVERRIDDEN_ACPI_TABLE',
-                   'TAINT_WARN',
-                   'TAINT_CRAP',
-                   'TAINT_FIRMWARE_WORKAROUND',
-                   'TAINT_OOT_MODULE',
-                   'TAINT_UNSIGNED_MODULE',
-                   'TAINT_SOFTLOCKUP',
-                   'TAINT_LIVEPATCH',
-                   'TAINT_AUX',
-                   'TAINT_RANDSTRUCT',
+    TAINT_FLAGS = [
+        'TAINT_PROPRIETARY_MODULE',
+        'TAINT_FORCED_MODULE',
+        'TAINT_CPU_OUT_OF_SPEC',
+        'TAINT_FORCED_RMMOD',
+        'TAINT_MACHINE_CHECK',
+        'TAINT_BAD_PAGE',
+        'TAINT_USER',
+        'TAINT_DIE',
+        'TAINT_OVERRIDDEN_ACPI_TABLE',
+        'TAINT_WARN',
+        'TAINT_CRAP',
+        'TAINT_FIRMWARE_WORKAROUND',
+        'TAINT_OOT_MODULE',
+        'TAINT_UNSIGNED_MODULE',
+        'TAINT_SOFTLOCKUP',
+        'TAINT_LIVEPATCH',
+        'TAINT_AUX',
+        'TAINT_RANDSTRUCT',
     ]
 
     flags = []
@@ -1323,9 +1353,9 @@ def is_security_update(pkgname, pkgversion):
     try:
         p = _apt_cache[pkgname]
         if 'Debian-Security' in [o.label for o in
-                        p.versions[pkgversion].origins]:
+                                 p.versions[pkgversion].origins]:
             return True
-    except:
+    except Exception:
         pass
 
     # Check 3:
@@ -1350,7 +1380,7 @@ def is_security_update(pkgname, pkgversion):
 
                 if 'CVE-20' in ln.upper():
                     return True
-    except:
+    except Exception:
         pass
 
     # guess 'no security update, but normal stable update' by default
